"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AppGraphQl = void 0;
const server_1 = require("@apollo/server");
const express4_1 = require("@apollo/server/express4");
const cacheControl_1 = require("@apollo/server/plugin/cacheControl");
const drainHttpServer_1 = require("@apollo/server/plugin/drainHttpServer");
const default_1 = require("@apollo/server/plugin/landingPage/default");
const schema_1 = require("@graphql-tools/schema");
const body_parser_1 = require("body-parser");
const cm_libs_context_aware_error_1 = require("cm-libs-context-aware-error");
const cm_libs_context_aware_logger_1 = require("cm-libs-context-aware-logger");
const cors_1 = __importDefault(require("cors"));
const graphql_1 = require("graphql");
const ws_1 = require("graphql-ws/lib/use/ws");
const app_base_1 = require("./app-base");
const app_graphql_directive_1 = require("./app-graphql-directive");
const common_1 = require("./common");
class AppGraphQl extends app_base_1.AppBase {
    _schema;
    async init(params) {
        const { schema, resolvers, ...appBaseInitParams } = params;
        await super.init(appBaseInitParams);
        const transformedSchema = (0, app_graphql_directive_1.applyDirectiveTransformers)((0, schema_1.makeExecutableSchema)({
            typeDefs: [app_graphql_directive_1.directiveTypeDefs, schema],
            resolvers
        }));
        // Keep the full schema
        this._schema = (0, graphql_1.print)(schema);
        const serverConfig = {
            schema: transformedSchema,
            formatError: this.formatError,
            csrfPrevention: true,
            includeStacktraceInErrorResponses: this.devMode ? true : false,
            plugins: [
                (0, cacheControl_1.ApolloServerPluginCacheControl)({ defaultMaxAge: 1 }),
                // Proper shutdown for the HTTP server.
                (0, drainHttpServer_1.ApolloServerPluginDrainHttpServer)({ httpServer: this.httpServer })
            ],
            ...appBaseInitParams?.apolloOptions
        };
        this.apolloServer = new server_1.ApolloServer(serverConfig);
        if (this.devMode) {
            this.apolloServer.addPlugin((0, default_1.ApolloServerPluginLandingPageLocalDefault)({ footer: false }));
        }
        if (appBaseInitParams.enablePubSub) {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.INFO, () => 'PubSub enabled');
            // Save the returned server's info so we can shutdown this server later
            const serverCleanup = (0, ws_1.useServer)({ schema: transformedSchema }, this.wsServer);
            this.apolloServer.addPlugin(
            // Proper shutdown for the WebSocket server.
            {
                async serverWillStart() {
                    return {
                        async drainServer() {
                            await serverCleanup.dispose();
                        }
                    };
                }
            });
        }
        // for backward compatible
        if (!appBaseInitParams.manualStart) {
            await this.apolloServer.start();
            this.app.use('/graphql', (0, cors_1.default)(), (0, body_parser_1.json)(), (0, express4_1.expressMiddleware)(this.apolloServer, {
                context: async ({ req }) => {
                    return this.context(req);
                }
            }));
        }
        return this;
    }
    async start() {
        await this.apolloServer.start();
        this.app.use('/graphql', (0, cors_1.default)(), (0, body_parser_1.json)(), (0, express4_1.expressMiddleware)(this.apolloServer, {
            context: async ({ req }) => {
                return this.context(req);
            }
        }));
        return this;
    }
    context(req) {
        const accessInfo = (0, common_1.getAuthToken)(req)?.info;
        req['logExcluded'] = false;
        if (this.devMode) {
            if (req.body.operationName !== 'IntrospectionQuery') {
                (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.CRITICAL, () => {
                    const headers = JSON.stringify(req.headers, null, 2);
                    const body = JSON.stringify(req.body, null, 2);
                    return `TRACE REQ: ${req.method} ${req.url} HTTP/${req.httpVersion} @${accessInfo.headers?.remoteIp}\nHEADERS: ${headers}\nBODY: ${body}`;
                });
            }
        }
        else {
            (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.NORMAL, () => `REQ: ${req.method} ${req.baseUrl} HTTP/${req.httpVersion} @${accessInfo.headers?.remoteIp}`);
        }
        return {
            req,
            accessInfo: (0, common_1.getAuthToken)(req)?.info,
            cache: this.apolloServer.cache ?? null
        };
    }
    formatError(error) {
        const { originalError, ...extensions } = error.extensions;
        (0, cm_libs_context_aware_logger_1.log)(cm_libs_context_aware_logger_1.Level.ERROR, () => {
            const { reason } = originalError
                ? (0, cm_libs_context_aware_error_1.describeError)(originalError)
                : { reason: `${error.message}${extensions.stacktrace ? '\n' + JSON.stringify(extensions.stacktrace, null, 2) : ''}` };
            return `GraphQL ERROR @ ${error.path?.[0] || '<PATH>'} - ${error.extensions?.code}: ${reason || cm_libs_context_aware_error_1.errors.UNKNOWN}`;
        });
        const ret = {
            message: error.message,
            path: error.path,
            extensions
        };
        return ret;
    }
    get schema() {
        return this._schema;
    }
}
exports.AppGraphQl = AppGraphQl;
//# sourceMappingURL=app-graphql.js.map